#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdio.h>
#include <sys/ioctl.h>
#include <unistd.h>
#include <stdlib.h>
#include <linux/cdrom.h>

#include "args.h"
#include "basic_info.h"

int basicInfo(const cdstatus_args * cdsargs)
{
    int status, type, drive;

    drive = open(cdsargs->drivename, O_RDONLY | O_NONBLOCK);
    if (drive == -1)
    {
        printf("Error opening drive %s.\n", cdsargs->drivename);
        perror(NULL);
        exit(EXIT_FAILURE);
    }

    status = ioctl(drive, CDROM_DRIVE_STATUS);
    if (status == -1)
    {
        perror("Error getting drive status (while performing ioctl)");
        exit(EXIT_FAILURE);
    }

    switch (status)
    {
    case CDS_NO_INFO:
        (void)puts("No information is available (i.e. this function is not implemented on the drive.");
        exit(EXIT_SUCCESS);
    case CDS_NO_DISC:
        (void)puts("No disc is in the drive.");
        exit(EXIT_SUCCESS);
    case CDS_TRAY_OPEN:
        (void)puts("The drive tray is open (or no disc is inserted).  Please close it then retry.");
        exit(EXIT_SUCCESS);
    case CDS_DRIVE_NOT_READY:
        (void)puts("The drive is not ready.");
        exit(EXIT_SUCCESS);
    case CDS_DISC_OK:
        if (cdsargs->silent==0)
        {
            (void)puts("The drive reads disc ok. Attempting to retrieve disc info.");
        }
        break;
    default:
        printf("Error: Invalid status code returned: %d", status);
        exit(EXIT_FAILURE);
    }

    type = ioctl(drive, CDROM_DISC_STATUS);

    if (type == -1)
    {
        perror("Unexpected error reading disc info from drive, exiting");
        exit(EXIT_FAILURE);
    }

    switch (type)
    {
    case CDS_NO_INFO:
        (void)puts("Unable to read disc type info.  Function not implemented.");
        exit(EXIT_FAILURE);
    case CDS_NO_DISC:
        (void)puts("\"No disc in drive\" status returned.  Unusual since disc presence and drive readiness was just verified.\n");
        exit(EXIT_FAILURE);
    case CDS_AUDIO:
        if (cdsargs->silent==0)
        {
            (void)puts("Disc appears to be an audio cd.");
        }
        break;
    case CDS_DATA_1:
        (void)puts("Disc appears to be a DATA MODE 1 cd.");
        break;
    case CDS_DATA_2:
        (void)puts("Disc appears to be a DATA MODE 2 cd.");
        break;
    case CDS_XA_2_1:
        (void)puts("Disc appears to be of type XA 2 1");
        break;
    case CDS_XA_2_2:
        (void)puts("Disc appears to be of type XA 2 2");
        break;
    case CDS_MIXED:
        if (cdsargs->silent==0)
        {
            (void)puts("Disc appears to be MIXED MODE. Will try to gather info on contained audio tracks.\n");
        }
        break;
    default:
        printf("Unknown type returned: %d.  Possible error occured.", type);
    }
    (void)close(drive);
    return type;
}
