
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <unistd.h>

#include "cddb_fetch_and_parse_results.h"
#include "cdstatus_cddb.h"
#include "cddb_hearken.h"

int fetchAndParseResults(void *buffer, char *category, unsigned int *discid, int client_sock)
{
	char *token;
	char *token2;
	unsigned int index;
	int result;

	(void)snprintf((char *) buffer, BUFFSIZE-1, "cddb read %s %08x\n", category, *discid);
	if(send(client_sock, buffer, strlen(buffer), 0)==-1)
	{
		perror("Error sending data to server, terminating");
		return 0;
	}
	result = hearken(client_sock, buffer);
	switch (result)
	{
	case 401:
	   	(void)puts("The server reports that the entry does not exist, despite having just listed it.  Please try a different server.");
	   	return 0;
	case 402:
		(void)puts("General server error.  Please try a different server.");
	   	return 0;
	case 403:
	   	(void)puts("The server reports that this database entry is corrupted.  Please try a different server.");
	   	return 0;
	case 409:
	   	(void)puts("The server indicates that the client has not established a proper server connection, despite having done so.  Please try a different server.");
	   	return 0;
	case 210:
		break;
	default:
	   	(void)puts("The server has returned an error code that does not comply with the published cddb protocol.  Please try a different server.");
		return 0;
	}

	/* parse and handle the returned information */
	/* discard the already used status line */
	token = strtok((char *) buffer, "\n");
	token = strtok(NULL, "\n");
	while (token != NULL)
	{
		if (token[0] == '#')
		{
			token = strtok(NULL, "\n");
			continue;
		}
		else if (token[0] == '.')
		{
			break;
		}

#ifdef DEBUG
		printf("Lineitem: %s\n", token);

#endif /*  */
		/* actual handling of data lines */
		if (strncmp(token, "DTITLE=", 7)==0)
		{

			/* artist and album names */
			token2 = strchr(token, '/');
			if(token2==NULL)
			{
				(void)puts("Invalid server response or error in parsing, exiting.");
				exit(EXIT_FAILURE);
			}
			--token2;
			strncpy(artist_name, token + 7, (size_t)(token2 - (token + 7)));

/*		 printf("Artist name: %s\n",artist_name);*/
			token2 += 3;
			strncpy(album_name, token2, strlen(token2) - 1);
			if (strchr(album_name, '\n') != NULL)
			{
				*strchr(album_name, '\n') = '\0';
			}

/*		 printf("Album name: %s\n",album_name);*/
		}

		else if (strncmp(token, "DYEAR=",6)==0)
		{
			/* Album year */
			if(sscanf(token+6,"%d",&album_year)!=1)
			{
				(void)puts("Error parsing album_year string (invalid server response).  Exiting.");
				exit(EXIT_FAILURE);
			}
		}

		/*else if(strncmp(token, "DGENRE=",7)==0)
		{
			memset(album_genre,'\0', 127);
			strncpy(album_genre,token+7,128);
		}*/

		else if (strncmp(token, "TTITLE", 6)==0)
		{

			/* track listing data */
			++cddb_tracks;
			if(sscanf(token, "TTITLE%u=", &index)!=1)
			{
				(void)puts("Invalid server response or parsing error, exiting.");
				exit(EXIT_FAILURE);
			}
			++index;
			if (strchr(token, '/') == NULL)
			{
				strncpy(trackinfo[index].title, strchr(token, '=') + 1, strlen(strchr(token, '=') + 1) - 1);

				/*printf("Track %d: %s\n",index,trackinfo[index].title); */
			}

			else
			{
				token2 = strchr(token, '/');
				--token2;
				strncpy(trackinfo[index].artist, strchr(token, '=') + 1, (size_t)(token2 - (strchr(token, '=') + 1)));
				token2 += 3;
				strncpy(trackinfo[index].title, token2, strlen(token2) - 1);

				/*printf("Track %d: %s by %s",index,trackinfo[index].title, \
				   trackinfo[index].artist); */
				(void)fflush(stdout);
			}
		}
		token = strtok(NULL, "\n");
	}
	strcpy((char *) buffer, "QUIT\n");
	if(send(client_sock, buffer, strlen(buffer), 0)==-1)
	{
		perror("Error sending data to server, exiting");
		return 0;
	}
	(void)close(client_sock);
	return 1;
}
