
#include <stdlib.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <stdio.h>
#include <string.h>

#include "cddb_handshake.h"
#include "cddb_hearken.h"
#include "cdstatus_cddb.h"

#ifdef HAVE_CONFIG_H
#include "config.h"
#else
#define VERSION "unknown"
#endif

int cddbHandshake(int client_sock, void* buffer)
{
	int result;
	int current;
	int supported;

	/*Environment variables*/
	char *env_user;
	char *env_hostname;

	result = hearken(client_sock, buffer);
	if ((result != 200) && (result != 201))
	{
		(void)puts("CDDB server is either busy (max users) or undergoing maintenance.  Please try a different server, or try later.");
		return 0;
	}
	memset(buffer, 0, BUFFSIZE);
	env_user=getenv("USER");
	env_hostname=getenv("HOSTNAME");
	(void)snprintf((char *) buffer, BUFFSIZE-1, "cddb hello %s %s cdstatus %s\n", env_user ? env_user : "unknown", env_hostname ? env_hostname : "unknown", VERSION);
	if(send(client_sock, buffer, strlen(buffer), 0)==-1)
	{
		perror("Error sending data to server, exiting");
		return 0;
	}
	result = hearken(client_sock, buffer);
	if (result != 200)
	{
		(void)puts("Error while \"handshaking\" with server.  If problem persists please try a different server.  If problem still persists, please contact the developer.");
		return 0;
	}
	memset(buffer, 0, BUFFSIZE);
	strcpy((char *) buffer, "proto\n");
	if(send(client_sock, buffer, strlen(buffer), 0)==-1)
	{
		perror("Error sending data to server, exiting");
		return 0;
	}
	result = hearken(client_sock, buffer);
	if (result != 200)
	{
		(void)puts("The server does not appear to handle the basic cddb protocol correctly.  Strongly recommend changing cddb servers.  Aborting.");
		return 0;
	}
	if(sscanf((const char *) buffer, "200 CDDB protocol level: current %d, supported %d", &current, &supported)!=2)
	{
		(void)puts("Error parsing cddb server response or invalid response. Exiting.");
		return 0;
	}

#ifdef DEBUG
	printf("Current protocol version: %d, max supported version: %d\n", current, supported);
#endif /*  */

	if (result < 5)
	{
		if(sscanf((const char *) buffer, "200 CDDB protocol level: current %*d, supported %d", &result)!=2)
		{
			(void)puts("Error parsing cddb server response or invalid response.");
			return 0;
		}

#ifdef DEBUG
		printf("Max protocol version: %d\n", result);
#endif /*  */
		if (result < 5)
		{
			(void)puts("Server does not support recent protocol updates.  Will continue, but new features will not be available.");
			return current;
		}

		else
		{
			memset(buffer, 0, BUFFSIZE);
			strcpy((char *) buffer, "proto 5\n");
			if(send(client_sock, buffer, strlen(buffer), 0)==-1)
			{
				perror("Error sending data to server, exiting");
				return 0;
			}
			result = hearken(client_sock, buffer);
			if (result != 201)
			{
				(void)puts("Error negotiating protocol level. Remaining at old level.");
				return current;
			}

#ifdef DEBUG
			else
			{
				(void)puts("Protocol changed to 5.");
			}

#endif /*  */
		}
	}
	return 5;
}

