
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/socket.h>

#include "cddb_perform_lookup.h"
#include "cddb_hearken.h"
#include "cdstatus_cddb.h"

int cddbPerformLookup(int client_sock, void * buffer, void * scratchbuffer, unsigned int discid, unsigned int tracks, const cd_toc_info * cdtocinfo, char *category, unsigned int first_inexact)
{
	char *token;
	int result;
	unsigned int index;

	memset(buffer, 0, BUFFSIZE);
	(void)snprintf((char *) buffer, BUFFSIZE-1, "cddb query %08x %u", discid, tracks);
	for (index = 1; index <= tracks; ++index)
	{
		(void)snprintf(scratchbuffer, SCRATCHBUFFER_SIZE-1, " %u", cdtocinfo[index].frame_global);
		strcat((char *) buffer, scratchbuffer);
	} 
	(void)snprintf(scratchbuffer, SCRATCHBUFFER_SIZE-1, " %u\n", cdtocinfo[0].frame_global / 75);
	strcat((char *) buffer, scratchbuffer);
	if(send(client_sock, buffer, strlen(buffer), 0)==-1)
	{
		perror("Error sending data to server, exiting");
		return 0;
	}
	result = hearken(client_sock, buffer);
	switch (result)
	{
	case 200:
		printf("Found exact match: %s\n", (char *) buffer);
		token = strtok((char *) buffer, " ");
		token = strtok(NULL, " ");
		if(token==NULL)
		{
			(void)puts("Received invalid response from cddb server, aborting.\n");
			return 0;
		}
		strncpy(category, token, 64);
		token = strtok(NULL, " ");
		if(token==NULL)
		{
			(void)puts("Error, invalid response received from cddb server");
			return 0;
		}
		if(sscanf(token, "%x", &discid)!=1)
		{
			(void)puts("Error parsing cddb server response or invalid response. Exiting.");
			return 0;
		}
		break;
	case 211:
		(void)puts("Found several inexact matches.");
		if (first_inexact==0)
		{
			(void)puts("Please select a number from the list below.");
		}
		else
		{
			(void)puts("Defaulting to first match.");
		}
		break;
	case 210:
		(void)puts("Found more than one exact match.");
		if (first_inexact==0)
		{
			(void)puts("Please select a number from the list below.");
		}
		else
		{
			(void)puts("Defaulting to first match.");
		}
		break;
	case 202:
		(void)puts("No match was found.  Please note that cdstatus does not currently support cddb entry submission.  Exiting.");
		return 0;
	case 403:
		(void)puts("The server reports that this database entry is corrupt. This is an ugly and unfortunate server-side error.  Exiting");
		return 0;
	case 409:
		(void)puts("The server is being either forgetful or haughty.  Then again, it's probably just broken.  Try a different server.");
		return 0;
	default:
		(void)puts("The server has responded with an invalid/undocumented response code that is not part of the spec.  Exiting.");
		return 0;
	}
	return result;
}
