#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifndef S_SPLINT_S
#include <stdio.h>
#include <errno.h>
#include <string.h>
#include <linux/cdrom.h>
#endif

#include <sys/ioctl.h>
#include "cd_toc_info.h"
#include "read_toc.h"

#ifndef CDROM_MSF
#define CDROM_MSF 0x02
#endif

#ifndef CDROMREADTOCHDR
#define CDROMREADTOCHDR 0x5305
#endif

#ifndef CDROMREADTOCENTRY
#define CDROMREADTOCENTRY 0x5306
#endif

#ifndef CD_FRAMES
#define CD_FRAMES 75
#endif

#ifndef CDROM_DATA_TRACK
#define CDROM_DATA_TRACK 0x04
#endif

#ifndef CDROM_LEADOUT
#define CDROM_LEADOUT 0xAA
#endif

#ifndef CDROMVOLREAD
#define CDROMVOLREAD 0x5313
#endif

int readTOC(int drive, cd_toc_info * cdtocinfo, struct cdrom_tochdr *cdtochdr, int silent)
{
	struct cdrom_tocentry cdtocentry;
	struct cdrom_volctrl cdvolume;
	unsigned int counter;
	int status;

	memset(&cdtocentry, 0, sizeof(cdtocentry));
	memset(&cdvolume, 0, sizeof(cdvolume));

	if (silent==0)
	{
		(void)puts("Reading table of contents.");
	}
	status = ioctl(drive, CDROMREADTOCHDR, cdtochdr);
	if (status == -1)
	{
		perror("Unable to read TOC");
		return errno;
	}

	if (silent==0)
	{
		printf("Disc appears to have %d tracks. Querying for track info.\n", cdtochdr->cdth_trk1);
	}

	for (counter = cdtochdr->cdth_trk0; counter <= cdtochdr->cdth_trk1; ++counter)
	{
		cdtocentry.cdte_format = CDROM_MSF;
		cdtocentry.cdte_track = counter;
		status = ioctl(drive, CDROMREADTOCENTRY, &cdtocentry);
		if (status == -1)
		{
			printf("Error querying track info for track %u\n", counter);
			perror(NULL);
			return errno;
		}
		else
		{
			cdtocinfo[counter].min = cdtocentry.cdte_addr.msf.minute;
			cdtocinfo[counter].sec = cdtocentry.cdte_addr.msf.second;
			cdtocinfo[counter].frame_local = cdtocentry.cdte_addr.msf.frame;
			cdtocinfo[counter].frame_global = cdtocinfo[counter].frame_local;
			(cdtocinfo[counter].frame_global) += ((cdtocinfo[counter].min) * CD_FRAMES * 60);
			(cdtocinfo[counter].frame_global) += ((cdtocinfo[counter].sec) * CD_FRAMES);
			/*cdtocinfo[counter].frame_global-=CD_MSF_OFFSET; */
			cdtocinfo[counter].data = (cdtocentry.cdte_ctrl & CDROM_DATA_TRACK);

			if (silent==0)
			{
				printf("Track %u starts at: minute: %u, second: %u, frame: %u; global frame: %u\n", counter, cdtocinfo[counter].min, cdtocinfo[counter].sec, cdtocinfo[counter].frame_local, cdtocinfo[counter].frame_global);
			}
		}
	}

	if (silent==0)
	{
		(void)puts("Gathering info on leadout track.");
	}
	cdtocentry.cdte_format = CDROM_MSF;
	cdtocentry.cdte_track = CDROM_LEADOUT;

	status = ioctl(drive, CDROMREADTOCENTRY, &cdtocentry);
	if (status == -1)
	{
		perror("Error querying track info for leadout track");
		return errno;
	}
	else
	{
		cdtocinfo[0].min = cdtocentry.cdte_addr.msf.minute;
		cdtocinfo[0].sec = cdtocentry.cdte_addr.msf.second;
		cdtocinfo[0].frame_local = cdtocentry.cdte_addr.msf.frame;
		cdtocinfo[0].frame_global = cdtocinfo[0].frame_local;
		(cdtocinfo[0].frame_global) += ((cdtocinfo[0].min) * CD_FRAMES * 60);
		(cdtocinfo[0].frame_global) += ((cdtocinfo[0].sec) * CD_FRAMES);
		/*cdtocinfo[0].frame_global-=CD_MSF_OFFSET; */

		if (silent==0)
		{
			printf("Leadout track starts at: minute: %u, second: %u, frame: %u; global frame: %u\n", cdtocinfo[0].min, cdtocinfo[0].sec, cdtocinfo[0].frame_local, cdtocinfo[0].frame_global);
		}
	}

	if (silent==0)
	{
		(void)puts("Querying drive volume settings:");
		status = ioctl(drive, CDROMVOLREAD, &cdvolume);
		if (status == -1)
		{
			perror("Unable to read volume settings");
			return errno;
		}
		(void)puts("Volume settings for drive:");
		printf("Channel 1: %d, Channel 2: %d, Channel 3: %d, Channel 4: %d\n", cdvolume.channel0, cdvolume.channel1, cdvolume.channel2, cdvolume.channel3);
	}

	return 0;
}
